// File    : MIBX2.CPP
// Author  : Eric Woodruff,  CIS ID: 72134,1150
// Updated : Wed 08/02/95 14:37:14
// Note    : Copyright 1993-95, Eric Woodruff, All rights reserved
// Compiler: Borland C++ 3.1 to 4.xx
//
// More functions for the Message/Input Box demo program.
//

#include <dos.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define Uses_TApplication
#define Uses_TButton
#define Uses_TDeskTop
#define Uses_TFileDialog
#define Uses_TKeys
#define Uses_TMenuBar
#define Uses_TProgram
#define Uses_TPXPictureValidator   // This is only pertinent when using TV 2.0
#define Uses_TResourceFile
#define Uses_TScreen
#define Uses_TStaticText
#define Uses_TStatusLine
#define Uses_TVCOLR                // Use this if you modified the TV.H file.
#define Uses_fpstream
#include <tv.h>

#if !defined(cpDefSize)
// Use this if you chose not to modify the Turbo Vision files.
#include <tvcolr.h>
#endif

#define Uses_TExplodeWindow
#include <texplode.h>

#include <msginpbx.h>

#include <tthermo.h>

#include "mibxdemo.h"

extern TPoint shadowSize;   // Altered when screen size changes.

extern TResourceFile *rsc;

extern char demo_cfg[MAXPATH];

extern short CmdLinePalette;

extern TextRecord  MsgDialogText;
extern ParamRecord MsgDialogOpts;

// ****************************************************************************

// Thermometer bar demo stuff.
// The '~' characters tell TV to highlight the surrounded text.
ThermoRecord ThermoOptions = { 12, 0, thTtlMsgBox, "~*~ Title Line ~*~",
    "~Test~ message", "55", "100", "10" };

char FillerTypes[19]  = {
                          pbfSolid, pbfEmpty, pbfTinyDots, pbfSmallDots,
                          pbfBigDots, pbfCircle1, pbfCircle2, pbfDashedLine1,
                          pbfDashedLine2, pbfDashedLine3, pbfDashedLine4,
                          pbfArrowHeads, pbfLightXHatch, pbfMedXHatch,
                          pbfDarkXHatch, pbfThinLine, pbfDoubleLine,
                          pbfThickLine1, pbfThickLine2 };

char *FillerNames[19] = {
      "Solid", "Empty", "Tiny Dots", "Small Dots", "Big Dots",
      "Circle 1", "Circle 2", "Dashed Line 1", "Dashed Line 2",
      "Dashed Line 3", "Dashed Line 4", "Arrowheads", "Light Crosshatch",
      "Medium Crosshatch", "Dark Crosshatch", "Thin Line", "Double Line",
      "Thick Line 1", "Thick Line 2" };

// ****************************************************************************
//
// Desktop initialization.
//
TDeskTop *TDemoApp::initDeskTop(TRect r)
{
    // Prevent drawing the desktop until the colors are set correctly.
    TProgram::application->setState(sfExposed, False);

    r.a.y++;
    r.b.y--;
    return new TDeskTop(r);
}

//
// Screen initialization.  This is overridden so that a command line
// specified color palette can be used.  If not, the default initScreen()
// will reset the palette based on monitor type every time it is called.
//
void TDemoApp::initScreen()
{
    TProgram::initScreen();

    // Force palette to equal the user-specified palette.
    if(CmdLinePalette > -1 && TApplication::appPalette != CmdLinePalette)
    {
        TApplication::appPalette = CmdLinePalette;

        // If monochrome, turn off shadows and turn on markers.
        if(CmdLinePalette == apMonochrome)
        {
            shadowSize.x = 0;
            shadowSize.y = 0;
            showMarkers = True;
        }
    }
}

//
// Menubar initialization.
//
TMenuBar *TDemoApp::initMenuBar(TRect)
{
    TMenuBar *menuBar = (TMenuBar *)rsc->get("MenuBar");

    // Set menubar width in case the application starts up in anything other
    // than a screen mode of 80x25 lines.
    menuBar->size.x = TScreen::screenWidth;

    return menuBar;
}

//
// Create statusline.
//
TStatusLine *TDemoApp::initStatusLine( TRect )
{
    TStatusLine *statLine = (TStatusLine *)rsc->get("StatusLine");

    // Set status line width and position in case the application starts up
    // in anything other than a screen mode of 80x25 lines.
    statLine->origin.y = TScreen::screenHeight - 1;
    statLine->size.x = TScreen::screenWidth;

    return statLine;
}

// Standard Out Of Memory error dialog.
void TDemoApp::outOfMemory(void)
{
    MsgBox(mdWarning, 1, mdCancelOnly, mdBeep | mdWarnColor,
        mdShortBeep, mdLowPitch,
        "Not enough memory for this operation.");
}

//
// Standard load config file.  Modify as needed.
//
void TDemoApp::loadConfig(Boolean UseFileSetting)
{
    fpstream *f = new fpstream(demo_cfg, ios::in | ios::nocreate | ios::binary);

    if(!f->good())
    {
        //  Turn on the screen?
        if(!(state & sfExposed))
        {
            setScreenMode(TScreen::screenMode);

#if _TV_VERSION == 0x0103
            if(TMouse::present())       // Adjust mouse limits if present.
                TMouse::setRange(TScreen::screenWidth - 1,
                    TScreen::screenHeight - 1);
#endif
        }

        MsgBox(mdError, 1, mdCancelOnly, mdTimeBeep | mdErrColor,
            5, mdShortBeep, mdLowPitch,
            "Could not open configuration file: ", demo_cfg);

        delete f;
        return;
    }

    ipstream &strm = *f;

    // Read palettes from the configuration file.
    short curr_palette = appPalette;
    for(short i = 0; i < apTotalPalettes; i++)
    {
        appPalette = i;
        TPalette *palette = &getPalette();
        strm.readBytes(palette->data, palette->data[0] + 1);
    }
    appPalette = curr_palette;

    // Get the video mode
    ushort scrMode;
    strm.readBytes(&scrMode, sizeof(scrMode));

    // Get the palette that was in use.
    short usePalette;
    strm.readBytes(&usePalette, sizeof(usePalette));

    // If not overridden from the command line, set the palette.
    if(CmdLinePalette == -1 || (CmdLinePalette != -1 && UseFileSetting))
        CmdLinePalette = usePalette;

    setScreenMode(scrMode);

#if _TV_VERSION == 0x0103
    if(TMouse::present())       // Adjust mouse limits if present.
        TMouse::setRange(TScreen::screenWidth - 1, TScreen::screenHeight - 1);
#endif
    // ************************************************************************
    // Add code here to load non-standard application-specific config data.

    delete f;
}

//
// Standard save config file.  Modify as needed.
//
void TDemoApp::saveConfig(void)
{
    fpstream *f = new fpstream(demo_cfg, ios::trunc | ios::binary);

    if(!f->good())
    {
        MsgBox(mdError, 1, mdCancelOnly, mdTimeBeep | mdErrColor,
            5, mdShortBeep, mdLowPitch,
            "Could not open configuration file: ", demo_cfg);

        delete f;
        return;
    }

    opstream &strm = *f;

    // Store the palettes
    short curr_palette = appPalette;
    for(short i = 0; i < apTotalPalettes; i++)
    {
        appPalette = i;
        TPalette *palette = &getPalette();
        strm.writeBytes(palette->data, palette->data[0] + 1);
    }
    appPalette = curr_palette;

    // Store current video mode
    strm.writeBytes(&TScreen::screenMode, sizeof(TScreen::screenMode));

    // Store current palette in use.
    strm.writeBytes(&appPalette, sizeof(appPalette));

    // ************************************************************************
    // Add code here to save non-standard application-specific config data.

    delete f;
}

// ****************************************************************************
// Other functions not in the standard front end file.

// Test formatted message lines.
void TDemoApp::FormatTest(void)
{
    int n10 = 10, n20 = 20, n30 = 30, n40 = 40, n50 = 50, n60 = 60;
    short s10 = 10;
    float f10 = 10.0, f20 = 20.0, f30 = 30.0, f40 = 40.0, f50 = 50.0;

    char ch = 'C';
    char str[10], lstr[10], *ptr;
    int len = 10, count;

    long lng = 75001L;
    double dbl = (double)101.0;
    long double lng_dbl = (long double)201.0;

    strcpy(str, "Test #3");
    strcpy(lstr, "(Test #4)");
    ptr = &str[0];

    // Note that you can combine one or more lines separated by
    // '\n' if you want to.
    //
    // It is EXTREMELY important that the number of parameters passed
    // match watch the format string specifies.
    // Otherwise, like printf(), you'll end up with garbage.
    //
    short code = MsgBox(mdInformation, 6, mdOKOnly, mdPlain | mdInfoColor,
        "%02d %3hd %i %o %u %#x %#X", 1, 1, 2, 3, 4, 5, 6,            // 1
        "%d %hd %i %o %u %#x %#X", n10, s10, n20, n30, n40, n50, n60, // 2

        "%f %e %g %E %G\n%7.3f %e %g %E %G", 10.0, 20.0, 30.0, 40.0, 50.0,
            f10, f20, f30, f40, f50,                                  // 3

        "%c %s %*s", 'C', "Test #1", 10, "(Test #2)",                 // 4
        "%c %-20s %-*s %n %p %p", ch, str, len, lstr, &count,
            &count, ptr,                                              // 5

        "%ld %lf %10.4Lf\n%ld %lf %Lf", 75000L, (double)100.0,        // 6
            (long double)200.0, lng, dbl, lng_dbl);

    // See MSGINPBX.H for the meaning of any error code.
    if(code < 0)
        MsgBox(mdError, 1, mdCancelOnly, mdTimeBeep | mdErrColor, 20,
            mdShortBeep, mdLowPitch, "Error #%d opening dialog box!", code);
}

void TDemoApp::ExpWinDemo(void)
{
    randomize();

    // Randomly select an initial size and position.
    TRect r(0, 0, 20 + rand() % 53, 6 + rand() % 16);

    r.move(rand() % 30, rand() % 10);

    TExplodeWindow *xw = new TExplodeWindow(r, "Demo Exploding Window", 0);
    xw->MakeItExplode(rand() % 50);     // Random delay.

    // Make it tileable and let it grow and shrink with the screen mode.
    xw->options |= ofTileable;
    xw->growMode = gfGrowAll | gfGrowRel;

    r = xw->getExtent();
    r.grow(-1, -1);

    TStaticText *s = new TStaticText(r, "This is a test.");
    s->growMode = gfGrowHiX | gfGrowHiY;
    xw->insert(s);

#if _TV_VERSION != 0x0103
    insertWindow(xw);
#else
    deskTop->insert(xw);
#endif
}

// Set the thermometer options and try them out.
void TDemoApp::BarDemo1(void)
{
    TDialog *pd = (TDialog *)rsc->get("OptionsDialog");

    if(validView(pd))
    {
        pd->setData(&ThermoOptions);

        // Stay here until cancelled.
        while(deskTop->execView(pd) != cmCancel)
        {
            pd->getData(&ThermoOptions);

            TDialog *demod = makeThermoTestDialog();

            if(validView(pd))
            {
                deskTop->execView(demod);
                destroy(demod);
            }
        }
        destroy(pd);
    }
}

// Creates a TWindow and inserts some thermometer bars that are only
// visual indicators (they don't have a process attached).
void TDemoApp::BarDemo2(void)
{
    TWindow *win = new TWindow(TRect(0,1,80,16), "TThermometer Demo", 0);
    win->options = ofCentered;

    TStaticText *s = new TStaticText(TRect(23, 2, 60, 4),
      "Time thermometers (HH:MM:SS.hh)\nPress a key or click mouse to close");
    s->options = ofCenterX;
    win->insert(s);

    s = new TStaticText(TRect(9, 6, 11, 13), "HH\n \nMM\n \nSS\n \nhh");
    win->insert(s);

    // 1 block = 1 unit. (0-23 hours, 0-59 minutes/seconds)
    TThermometer *hbar = new TThermometer(TRect(12, 5, 35, 7), NULL, 23, thMsgAbove);
    TThermometer *mbar = new TThermometer(TRect(12, 7, 71, 9), NULL, 59, thMsgAbove);

    // Add a little variety to the bar types.
    TThermometer *sbar = new TThermometer(TRect(12, 9, 71, 11), NULL, 59,
        thMsgAbove, pbfDarkXHatch, pbfCircle1);

    // Can't fit 99 blocks on the screen.  Approximate scale = 2:1.
    TThermometer *tbar = new TThermometer(TRect(12, 11, 62, 13), NULL, 99,
        thMsgAbove, pbfTinyDots, pbfBigDots);

    win->insert(hbar);
    win->insert(mbar);
    win->insert(sbar);
    win->insert(tbar);

    deskTop->insert(win);

    struct time tm;    // Simply update each bar with its part of the
                       // time until a key is pressed or mouse button clicked.
    char str[50];

    TEvent event;
    event.what = evNothing;

    while(event.what != evKeyboard && event.what != evMouseDown)
    {
        if(eventAvail())
            getEvent(event);

        gettime(&tm);

        // These don't have a thermFunc, they are manipulated directly.
        sprintf(str, "%d", tm.ti_hour);
        hbar->setMessage(str);
        hbar->dispCurIter((ulong)tm.ti_hour);

        sprintf(str, "%d", tm.ti_min);
        mbar->setMessage(str);
        mbar->dispCurIter((ulong)tm.ti_min);

        sprintf(str, "%d - %3.2lf%%", tm.ti_sec, sbar->percentDone());
        sbar->setMessage(str);
        sbar->dispCurIter((ulong)tm.ti_sec);

        // The hundredths will always be random.  You might not always see
        // this bar hit the end of the scale before the next second arrives.
        sprintf(str, "%d", tm.ti_hund);
        tbar->setMessage(str);
        tbar->dispCurIter((ulong)tm.ti_hund);
    }

    destroy(win);       // Calls destructors for the TThermometers too.
}

// Reads a TWindow and some thermometer bars from a stream that are only
// visual indicators (they don't have a process attached).
void TDemoApp::BarDemo3(void)
{
    // Restore objects from the resource file.
    TWindow *win = (TWindow *)rsc->get("TWindowDemo");

    if(!win)
        return;

    TThermometer *hbar = (TThermometer *)rsc->get("THourBar");
    TThermometer *mbar = (TThermometer *)rsc->get("TMinsBar");
    TThermometer *sbar = (TThermometer *)rsc->get("TSecsBar");
    TThermometer *tbar = (TThermometer *)rsc->get("THundBar");

    win->insert(hbar);      // Put them in the window.
    win->insert(mbar);
    win->insert(sbar);
    win->insert(tbar);

    deskTop->insert(win);

    struct time tm;    // Simply update each bar with its part of the
                       // time until a key is pressed or mouse button clicked.
    char str[50];

    TEvent event;
    event.what = evNothing;

    while(event.what != evKeyboard && event.what != evMouseDown)
    {
        if(eventAvail())
            getEvent(event);

        gettime(&tm);

        // These don't have a thermFunc, they are manipulated directly.
        sprintf(str, "%d", tm.ti_hour);
        hbar->setMessage(str);
        hbar->dispCurIter((ulong)tm.ti_hour);

        sprintf(str, "%d", tm.ti_min);
        mbar->setMessage(str);
        mbar->dispCurIter((ulong)tm.ti_min);

        sprintf(str, "%d - %3.2lf%%", tm.ti_sec, sbar->percentDone());
        sbar->setMessage(str);
        sbar->dispCurIter((ulong)tm.ti_sec);

        // The hundredths will always be random.  You might not always see
        // this bar hit the end of the scale before the next second arrives.
        sprintf(str, "%d", tm.ti_hund);
        tbar->setMessage(str);
        tbar->dispCurIter((ulong)tm.ti_hund);
    }

    destroy(win);       // Calls destructors for the TThermometers too.
}

// ****************************************************************************

// The only reason this function exists is for the test program.  Under
// all other circumstances, you will just create a TThermometer bar as
// you normally would in any other dialog box.  This is used simply to
// put the class through its paces by letting you interact with another
// dialog box that sets the options.
TDialog *makeThermoTestDialog(void)
{
    char str[80];

    TDialog* dlg = new TDialog(TRect(0,0,61,18), "Demo TThermometer Bar");
    if(!dlg)
        return NULL;

    dlg->options |= ofCentered;

    TStaticText *s = new TStaticText(TRect(4,9,58,10),
        "When running, press a key or click mouse to stop.");
    dlg->insert(s);

    sprintf(str, "Background Character: %s", FillerNames[ThermoOptions.backGrnd]);
    s = new TStaticText(TRect(4,10,58,11), str);
    dlg->insert(s);

    sprintf(str, "Foreground Character: %s", FillerNames[ThermoOptions.foreGrnd]);
    s = new TStaticText(TRect(4,11,58,12), str);
    dlg->insert(s);

    // NOTE: YOU MUST SET THE bfBroadcast BIT.  OTHERWISE, THE THERMOMETER
    //       WON'T SEE THE BUTTON PRESS!
    TButton *btn = new TButton(TRect(12,14,28,16), "~S~tart Test", cmOK,
        bfNormal | bfBroadcast);
    dlg->insert(btn);

    btn = new TButton(TRect(38,14,48,16), "Cancel", cmCancel, bfNormal);
    dlg->insert(btn);

    short width = atoi(ThermoOptions.ThermWidth);
    if(width < 1)
        width = 1;  // Simple edits for the demo.

    if(width > 55)
        width = 55;

    // Convert demo options to their proper types.
    ulong  Iters  = strtoul(ThermoOptions.Iterations, NULL, 10);
    uchar  bkChar = FillerTypes[ThermoOptions.backGrnd];
    uchar  fgChar = FillerTypes[ThermoOptions.foreGrnd];
    short  Delay  = atoi(ThermoOptions.Delay);

    // All but the extra bit from the check boxes.
    ushort Options = ThermoOptions.Options & ~DoPause;

    if(Delay < 0)
        Delay = 0;

    //
    // DemoProcess is attached as the function that handleEvent() will call.
    //
    TMyThermometer *bar = new TMyThermometer( TRect(1, 2,
      width + 1 + (((Options) & thIsBoxed) ? 2 : 0),
      3 + (((Options) & thHasTitle)  ? 1 : 0) +
          (((Options) & thHasMsg)    ? 1 : 0) +
          (((Options) & thIsBoxed)   ? 2 : 0)),
        DemoProcess, Iters, Options, bkChar, fgChar );

    bar->options |= ofCenterX;          // Center horizontally.

    bar->setDelay(Delay);               // Delay between iterations.
    bar->setPause(Boolean(ThermoOptions.Options & DoPause));   // Pause if told to.

    // Won't be display unless the thHasTitle bit is set in THOptions.
    bar->setTitle(ThermoOptions.TitleText);

    // Won't be displayed unless the thHasMsg bit is set in THOptions.
    bar->setMessage(ThermoOptions.MessageText);
    dlg->insert(bar);

    dlg->selectNext(False);     // Always move off of the bar if it was
                                // inserted last.
    return dlg;
}

// ****************************************************************************
// Demonstrate the process function for the Filler Types test.
//
#pragma argsused        // Owner isn't used for this demo.
void DemoProcess(void *Owner, void *thBar)
{
    TMyThermometer *bar = (TMyThermometer *)thBar;

    char FirstMsg[256];
    TEvent event;

    strcpy(FirstMsg, bar->getMsgTxt());      // Save original message.

    // If not set with the constructor, determine the total number of
    // iterations to perform and set it with setTotIter().
    //
    // x = number of records, items, loops, ...
    // bar->setTotIter(x);

    // Remember to use '<=' if necessary to go all the way through
    // the last iteration!
    for(ulong count = 1L; count <= bar->getTotIter(); count++)
    {
        // Here is where the process occurs.
        //      .
        //      .

        // Set the message to display.  If you want any count information
        // to be valid in the display, set the message BEFORE
        // calling dispCurIter().  dispCurIter() is what actually
        // draws the message and bar.  If you don't need to update the
        // percentage bar, simply call drawView() (see below).
        bar->setMessage("Iteration #: %lu", count);

        bar->dispCurIter(count);    // Update the percentage done bar.
        delay(bar->Delay);          // Short pause for demo.

        if(bar->pauseHalfway && count == bar->getTotIter() / 2)
        {
            bar->setMessage("* * Another Message * *");
            bar->drawView();    // No change in iteration, just the message.

            MsgBox(mdInformation, 1, mdOKOnly, mdNotifyColor,
                "Halfway to completion");
        }

        if(TProgram::deskTop->eventAvail())
        {
            // Cancel early?
            TProgram::deskTop->getEvent(event);
            if(event.what == evKeyboard || event.what == evMouseDown)
            {
                bar->setMessage("~* Demo process cancelled *~");
                bar->drawView();

                delay(500);
                count = bar->getTotIter();
            }
        }
    }
    MsgBox(mdInformation, 1, mdOKOnly, mdNotifyColor, "All Done");

    bar->setMessage(FirstMsg);       // Reset original message.
                                     // It'll get drawn again by cleanUp().
}

// ****************************************************************************
// Demonstration function for the streamed TThermometer bar.
// A pointer to the owning view is passed as well as a pointer to the actual
// thermometer bar.  That way, you can manipulate the owner or get information
// from it too.
void FileSearch(void *Owner, void *thBar)
{
    TEvent event;
    ffblk finfo;
    int done;
    ulong count = 0L, i;

    StreamDataRec TData;    // Data structure must match the dialog box!

    TDialog *dlg = (TDialog *)Owner;
    TThermometer *bar = (TThermometer *)thBar;

    // Is it the right dialog?
    if(!strcmp(dlg->getTitle(0), "Demo TThermometer Bar"))
        dlg->getData(&TData);           // Get the dialog's data.
    else
        TData.FSpec[0] = EOS;    // Check for another dialog box's data or ...

    // Append default filespec.
    if(!strchr(TData.FSpec, '*') && !strchr(TData.FSpec, '?'))
        strcat(TData.FSpec, "\\*.*");

    // Work with the data obtained from the dialog box.
    done = findfirst(TData.FSpec, &finfo, FA_RDONLY | FA_HIDDEN |
        FA_SYSTEM | FA_LABEL | FA_DIREC | FA_ARCH);

    // Get total directory entries.
    while(!done)
    {
        count++;
        bar->setMessage("%lu entries in %s", count, TData.FSpec);
        bar->drawView();

        done = findnext(&finfo);
    }

    bar->setTotIter((ulong)count);
    findfirst(TData.FSpec, &finfo, FA_RDONLY | FA_HIDDEN |
        FA_SYSTEM | FA_LABEL | FA_DIREC | FA_ARCH);

    for(i = 1; i <= count; i++)
    {
        bar->setMessage("%3lu: %s", i, finfo.ff_name);
        bar->dispCurIter(i);      // Nothing fancy.  Just count to the total.
        delay(50);                // Could be a start for a file copier or ...

        if(TProgram::deskTop->eventAvail())
        {
            // Cancel early?
            TProgram::deskTop->getEvent(event);
            if(event.what == evKeyboard || event.what == evMouseDown)
            {
                bar->setMessage("~* Demo process cancelled *~");
                bar->drawView();
                i = bar->getTotIter();
            }
        }
        findnext(&finfo);
    }
    delay(500);
    bar->setMessage("");
}

// ****************************************************************************

// The only reason this function exists is for the test program.  Under
// all other circumstances, you will just create a message or input box
// as you normally would (call the MsgBox() or InpBox() function).  This is
// used simply to put the make function through its paces by letting you
// interact with another dialog box that sets the options.  With the variable
// parameter list, there is a seemingly infinite number of ways to use the
// make function so this simplifies matters greatly.

#if !defined(__FLAT__)

// 16-bit version

TDialog *makeMIBXTestDialog(void)
{
#if _TV_VERSION == 0x0200
    // Any validator type can be used.  This just lets us alter the picture
    // for testing via the dialog options.
    TPXPictureValidator *val;
#endif

    TDialog *dlg;
    int   NumButtons, NumLines, BeepFreq, BeepDuration, TimeOutLen,
          ButtonCnt, LineLength, HasHistory, HasValidator, ErrorCode,
          i, TotParms = 0, Opts = mdPlain;
    char *p;

    // Do some character conversion.  For testing allow "\n" to get
    // converted to a literal '\n' character and "\003" to get converted
    // to a literal '\003' character.
    for(i = 0; i < 16; i++)
        for(p = MsgDialogText.Lines[i]; *p; p++)
            if(*p == '\\' && *(p + 1) == 'n')
            {
                *p = '\n';
                strcpy(p + 1, p + 2);
            }
            else
                if(!strncmp(p, "\\003", 4))
                {
                    *p = '\003';
                    strcpy(p + 1, p + 4);
                }

    // Get explode/implode setting.
    Opts |= ((MsgDialogOpts.Options & 0x01) << 5);

    // Get the password setting.
    Opts |= ((MsgDialogOpts.Options & 0x04) << 10);

    // Get the help context setting
    Opts |= ((MsgDialogOpts.Options & 0x10) << 9);

    // Disable sound effects?  If so, this will remain in effect for
    // the other areas of the application too.
    if(MsgDialogOpts.Options & 0x20)
        mibxNoSound = True;
    else
        mibxNoSound = False;

    // Get color setting.
    Opts |= (MsgDialogOpts.Color << 8);

    NumLines     = MsgDialogOpts.LineCount;
    NumButtons   = MsgDialogOpts.ButtonCount;
    HasHistory   = MsgDialogOpts.Options & 0x02;
    HasValidator = MsgDialogOpts.Options & 0x08;
    LineLength   = atoi(MsgDialogOpts.LineLen);
    BeepFreq     = atoi(MsgDialogOpts.Pitch);
    BeepDuration = atoi(MsgDialogOpts.BeepTime);
    TimeOutLen   = atoi(MsgDialogOpts.MaxTime);

    // Determine option settings.
    if(LineLength > 0)
    {
#if _TV_VERSION == 0x0200
        if(HasValidator)
            val = new TPXPictureValidator(MsgDialogText.Validator, True);
#endif
        if(HasHistory)
        {
            if(HasValidator)
                Opts |= mdHistVal;
            else
                Opts |= mdInputHis;
        }
        else
            if(HasValidator)
                Opts |= mdInputVal;
            else
                Opts |= mdInput;
    }

    if(BeepDuration > 0)
        Opts |= mdBeep;

    if(TimeOutLen > 0)
        Opts |= mdTime;

    if(NumButtons > 5)
    {
        NumButtons -= 5;                // Make it a valid count.
        ButtonCnt = NumButtons;         // Remember number for stack cleanup.
    }
    else
    {
        ButtonCnt = 0;
        NumButtons = 0 - NumButtons;    // Invert for default types.
    }

    // Push all the necessary parameters on the stack.

    if(Opts & mdHelpCtx)
    {
        asm     mov     ax, 54321   // Always 54321 for the demo.
        asm     push    ax

        TotParms++;
    }

asm {
    mov     cx, NumButtons
    jcxz    short TextData       // Skip if there are to be no buttons.
    test    ch, 0x80
    jnz     short TextData       // Skip if there are to be default buttons.

    mov     ax, TEXT_SIZE        // Point to the last button to push.
    dec     cx
    mul     cx
    inc     cx
    mov     bx, offset MsgDialogText.Buttons
    add     bx, ax
    mov     ax, bfNormal        // Options.  For demo, always bfNormal.
    mov     dx, 500
    add     dx, cx
    }

PushButtonData:

asm {
    push    ax
    push    dx                  // Command.  For demo, 500 + button number.
    push    ds
    push    bx                  // Button text
    dec     dx
    sub     bx, TEXT_SIZE
    loop    PushButtonData      // Keep going until no more buttons to push.
    }

TextData:

asm {
    mov     cx, NumLines
    jcxz    short OtherParms    // Skip if no text lines.

    mov     ax, TEXT_SIZE       // Point to last text line to push.
    dec     cx
    mul     cx
    inc     cx
    mov     bx, offset MsgDialogText.Lines
    add     bx, ax
    }

TextLineData:

asm {
    push    ds
    push    bx                  // Dialog line text.
    sub     bx, TEXT_SIZE
    loop    TextLineData
    }

OtherParms:

asm {
    xor     dx, dx
    mov     bx, Opts
    test    bx, mdInputVal
    jz      short Skip0
    }

#if _TV_VERSION == 0x0200
asm {
    mov     ax, word ptr [val + 2]
    push    ax
    mov     ax, word ptr [val]
    push    ax
    inc     dx
    inc     dx
    }
#else
asm {
    xor     ax, ax      // Validators are only supported under TV 2.0
    push    ax
    push    ax
    inc     dx
    inc     dx
    }
#endif

Skip0:

asm {
    test    bx, mdInputHis
    jz      short Skip1
    mov     ax, 1               // History ID of 1 for demo.
    push    ax
    inc     dx
    jmp     short PushInpLen    // Input length is implied.
    }
Skip1:

asm {
    test    bx, mdInputVal      // Input length is implied if there
    jnz     short PushInpLen    // is a validator too.

    test    bx, mdInput
    jz      short Skip2
    }
PushInpLen:

asm {
    mov     ax, LineLength
    push    ax
    inc     dx
    }
Skip2:

asm {
    test    bx, mdBeep
    jz      short Skip3
    mov     ax, BeepFreq
    push    ax
    mov     ax, BeepDuration
    push    ax
    inc     dx
    inc     dx
    }
Skip3:

asm {
    test    bx, mdTime
    jz      short Skip4
    mov     ax, TimeOutLen
    push    ax
    inc     dx
    }
Skip4:

    asm add     TotParms, dx    // Remember the total number pushed.

    // Call the function with the fixed parameters.
    dlg = makeMsgDialog(
        (MsgDialogText.TxtTitle[0]) ? MsgDialogText.TxtTitle : NULL,
            NumLines, NumButtons, ErrorCode, Opts);

asm {
    mov     ax, ButtonCnt       // Calculate number of bytes pushed onto
    mov     cl, 3               // the stack.
    sal     ax, cl
    mov     bx, NumLines
    dec     cl
    sal     bx, cl
    add     ax, bx
    mov     bx, TotParms
    add     bx, bx
    add     ax, bx
    add     sp, ax              // Clear the bytes from stack
    }

    // See MSGINPBX.H for the meaning of any error code.
    if(!dlg)
        MsgBox(mdError, 1, mdCancelOnly, mdTimeBeep | mdErrColor, 30,
            mdMediumBeep, mdLowPitch, "Error #%d opening dialog box!",
            ErrorCode);

    // Put things back the way they were.
    for(i = 0; i < 16; i++)
        for(p = MsgDialogText.Lines[i]; *p; p++)
            if(*p == '\n')
            {
                memmove(p + 2, p + 1, strlen(p + 1) + 1);
                *p++ = '\\';
                *p = 'n';
            }
            else
                if(*p == '\003')
                {
                    memmove(p + 4, p + 1, strlen(p + 1) + 1);
                    memcpy(p, "\\003", 4);
                }

    return dlg;
}

#else

// ****************************************************************************
// 32-bit version.

// These are necessary to prevent the optimizer from using ESP directly to
// reference the automatic variables.  It can't do that because __emit__()
// statements are used to push registers onto the stack.  The optimizer can't
// see that so it isn't aware that ESP is changing.  Disabling the
// optimizations makes it use EBP instead.
#pragma option -Z-
#pragma option -Od

TDialog *makeMIBXTestDialog(void)
{
    // Any validator type can be used.  This just lets us alter the picture
    // for testing via the dialog options.
    TPXPictureValidator *val;

    TDialog *dlg;
    int   NumButtons, NumLines, BeepFreq, BeepDuration, TimeOutLen,
          ButtonCnt, LineLength, HasHistory, HasValidator, ErrorCode,
          i, TotParms = 0, Opts = mdPlain;
    char *p;

    // Do some character conversion.  For testing allow "\n" to get
    // converted to a literal '\n' character and "\003" to get converted
    // to a literal '\003' character.
    for(i = 0; i < 16; i++)
        for(p = MsgDialogText.Lines[i]; *p; p++)
            if(*p == '\\' && *(p + 1) == 'n')
            {
                *p = '\n';
                strcpy(p + 1, p + 2);
            }
            else
                if(!strncmp(p, "\\003", 4))
                {
                    *p = '\003';
                    strcpy(p + 1, p + 4);
                }

    // Get explode/implode setting.
    Opts |= ((MsgDialogOpts.Options & 0x01) << 5);

    // Get the password setting.
    Opts |= ((MsgDialogOpts.Options & 0x04) << 10);

    // Get the help context setting
    Opts |= ((MsgDialogOpts.Options & 0x10) << 9);

    // Disable sound effects?  If so, this will remain in effect for
    // the other areas of the application too.
    if(MsgDialogOpts.Options & 0x20)
        mibxNoSound = True;
    else
        mibxNoSound = False;

    // Get color setting.
    Opts |= (MsgDialogOpts.Color << 8);

    NumLines     = MsgDialogOpts.LineCount;
    NumButtons   = MsgDialogOpts.ButtonCount;
    HasHistory   = MsgDialogOpts.Options & 0x02;
    HasValidator = MsgDialogOpts.Options & 0x08;
    LineLength   = atoi(MsgDialogOpts.LineLen);
    BeepFreq     = atoi(MsgDialogOpts.Pitch);
    BeepDuration = atoi(MsgDialogOpts.BeepTime);
    TimeOutLen   = atoi(MsgDialogOpts.MaxTime);

    // Determine option settings.
    if(LineLength > 0)
    {
        if(HasValidator)
            val = new TPXPictureValidator(MsgDialogText.Validator, True);

        if(HasHistory)
        {
            if(HasValidator)
                Opts |= mdHistVal;
            else
                Opts |= mdInputHis;
        }
        else
            if(HasValidator)
                Opts |= mdInputVal;
            else
                Opts |= mdInput;
    }

    if(BeepDuration > 0)
        Opts |= mdBeep;

    if(TimeOutLen > 0)
        Opts |= mdTime;

    if(NumButtons > 5)
    {
        NumButtons -= 5;                // Make it a valid count.
        ButtonCnt = NumButtons;         // Remember number for stack cleanup.
    }
    else
    {
        ButtonCnt = 0;
        NumButtons = 0 - NumButtons;    // Invert for default types.
    }

    // Push all the necessary parameters on the stack.

    if(Opts & mdHelpCtx)
    {
        _EAX = 54321;

        __emit__(0x50);     // PUSH EAX

        TotParms++;
    }

    // Push button info if there is to be any.
    if(NumButtons && !(NumButtons & 0x8000))
    {
        _ECX = NumButtons - 1;
        _EAX = _ECX * TEXT_SIZE;    // Point to last button to push.
        _ECX += 1;

        _EBX = ((unsigned long)&MsgDialogText.Buttons[0][0]) + _EAX;
        _EAX = bfNormal;            // Options. For demo, always bfNormal.
        _EDX = _ECX;
        _EDX += 500;

        do
        {
            __emit__(0x50);     // PUSH EAX, EDX, EBX.

            __emit__(0x52);     // Command.  For demo, 500 + button number.

            __emit__(0x53);     // Button text.

            _EBX -= TEXT_SIZE;
            _EDX--;
            _ECX--;

        } while(_ECX);
    }

    // Push line data, if any.
    if(NumLines)
    {
        _ECX = NumLines - 1;
        _EAX = _ECX * TEXT_SIZE;    // Point to last text line to push.
        _ECX += 1;

        _EBX = ((unsigned long)&MsgDialogText.Lines[0][0]) + _EAX;

        do
        {
            __emit__(0x53);         // PUSH EBX - Dialog line text.

            _EBX -= TEXT_SIZE;
            _ECX--;

        } while(_ECX);
    }

    if(Opts & mdInputVal)
    {
        _EAX = (unsigned long)val;         // Push validator.

        __emit__(0x50);     // PUSH EAX

        TotParms++;
    }

    if(Opts & mdInputHis)
    {
        __emit__(0x6A);     // PUSH large 1 - History ID of 1 for demo.
        __emit__(0x01);
        TotParms++;
    }

    // Push input length if any input options are specified.
    if(Opts & (mdInputVal | mdInputHis | mdInput))
    {
        _EAX = LineLength;

        __emit__(0x50);     // PUSH EAX

        TotParms++;
    }

    if(Opts & mdBeep)
    {
        _EAX = BeepFreq;
        _EBX = BeepDuration;

        __emit__(0x50);     // PUSH EAX, EBX
        __emit__(0x53);

        TotParms += 2;
    }

    if(Opts & mdTime)
    {
        _EAX = TimeOutLen;

        __emit__(0x50);     // PUSH EAX

        TotParms++;
    }

    // Call the function with the fixed parameters.
    dlg = makeMsgDialog(
        (MsgDialogText.TxtTitle[0]) ? MsgDialogText.TxtTitle : NULL,
            NumLines, NumButtons, ErrorCode, Opts);

    // Calculate the number of bytes pushed onto the stack.
    ButtonCnt *= 12;
    NumLines *= 4;
    TotParms *= 4;
    TotParms += ButtonCnt + NumLines;
    _EAX = TotParms;

    __emit__(0x03);         // ADD ESP, EAX - Clear the bytes from stack
    __emit__(0xE0);


    // See MSGINPBX.H for the meaning of any error code.
    if(!dlg)
        MsgBox(mdError, 1, mdCancelOnly, mdTimeBeep | mdErrColor, 30,
            mdMediumBeep, mdLowPitch, "Error #%d opening dialog box!",
            ErrorCode);

    // Put things back the way they were.
    for(i = 0; i < 16; i++)
        for(p = MsgDialogText.Lines[i]; *p; p++)
            if(*p == '\n')
            {
                memmove(p + 2, p + 1, strlen(p + 1) + 1);
                *p++ = '\\';
                *p = 'n';
            }
            else
                if(*p == '\003')
                {
                    memmove(p + 4, p + 1, strlen(p + 1) + 1);
                    memcpy(p, "\\003", 4);
                }

    return dlg;
}

#endif
