// System  : MsgInpBx 2.00 - Message/Input Box Library
// File    : TEXPLODE.H
// Author  : Eric Woodruff,  CIS ID: 72134,1150
// Updated : Wed 08/02/95 14:37:14
// Note    : Copyright 1993-95, Eric Woodruff, All rights reserved
// Compiler: Borland C++ 3.1 to 4.xx
//
// This header file defines a TWindow derived class and a TDialog derived
// class that can explode on opening or showing and implode on closing
// or hiding.  TExplodeDialog is the base class for the message dialog box
// which gives it the explode/implode option.
//

#if !defined(__FLAT__)

#if !defined(_NEAR)
#define _NEAR   near
#endif

#if !defined(_FAR)
#define _FAR    far
#endif

#endif

// Located in TGROUP.CPP and required by the changeBounds() functions.
extern void doCalcChange(TView *p, void *d);

// Explode/Implode states
#define expNeither      0
#define expExploding    1
#define expImploding    2

// Explode/Implode status
#define expWillExplode  0x0001
#define expWillImplode  0x0002

#if defined(Uses_TExplodeWindow) && !defined(__TExplodeWindow)
#define __TExplodeWindow

class _FAR TRect;
class _FAR TWindow;

//
// Class TExplodeWindow
//
//  Palette layout:
//      Same as TWindow
//
class TExplodeWindow : public TWindow
{
protected:
    short IsExplodingImploding, ExpImpStatus,   // State flags.
          msDelay;                              // Milliseconds between draws.

    void Explode(void);     // Explode the window upon opening/showing.
    void Implode(void);     // Implode the window upon closing/hiding.

public:
    TExplodeWindow(const TRect &r, const char *aTitle, short aNumber);

    virtual void setState(ushort aState, Boolean enable);
    virtual void changeBounds(const TRect &bounds);
    virtual void sizeLimits(TPoint &min, TPoint &max);
    virtual void draw(void);
    virtual void close(void);

    // Turn it back into a normal window.
    inline void MakeItNormal(void)
        {   ExpImpStatus = expNeither;  }

    // Tell the window to explode on opening and implode on closing.
    // Also used to change the explode/implode delay.
    inline void MakeItExplode(short ms, short ExpOrImp = expWillExplode)
    {
        if(ExpImpStatus == expNeither)
            ExpImpStatus = ExpOrImp;

        msDelay = ms;
    }

    // Get the current explode/implode delay factor.
    inline short GetDelay(void)
        {   return msDelay;     }

// This stuff will make it streamable.
private:
    virtual const char *streamableName() const
        { return name; }

protected:
    TExplodeWindow(StreamableInit) : TWindow(streamableInit),
        TWindowInit(streamableInit) {}
    virtual void write( opstream & );
    virtual void *read( ipstream & );

public:
    static const char * const _NEAR name;
    static TStreamable *build();
};

inline ipstream& operator >> ( ipstream& is, TExplodeWindow& cl )
    { return is >> (TStreamable&)cl; }
inline ipstream& operator >> ( ipstream& is, TExplodeWindow*& cl )
    { return is >> (void *&)cl; }

inline opstream& operator << ( opstream& os, TExplodeWindow& cl )
    { return os << (TStreamable&)cl; }
inline opstream& operator << ( opstream& os, TExplodeWindow* cl )
    { return os << (TStreamable *)cl; }

#endif      // TExplodeWindow

//
// Class TExplodeDialog
//
//  Palette layout:
//      Same as TDialog
//
#if defined(Uses_TExplodeDialog) && !defined(__TExplodeDialog)
#define __TExplodeDialog

class _FAR TRect;
class _FAR TDialog;
class _FAR TWindow;
class _FAR TWindowInit;

//
// Class TExplodeDialog
//
//  Palette layout:
//      Same as TDialog
//
class TExplodeDialog : public TDialog
{
protected:
    short IsExplodingImploding, ExpImpStatus,   // State flags.
          msDelay;                              // Milliseconds between draws.

    void Explode(void);     // Explode the window upon opening/showing.
    void Implode(void);     // Implode the window upon closing/hiding.

public:
    TExplodeDialog(const TRect &r, const char *aTitle);

    virtual void setState(ushort aState, Boolean enable);
    virtual void changeBounds(const TRect &bounds);
    virtual void sizeLimits(TPoint &min, TPoint &max);
    virtual void draw(void);
    virtual void close(void);

    // Turn it back into a normal dialog.
    inline void MakeItNormal(void)
        {   ExpImpStatus = expNeither;  }

    // Tell the dialog to explode on opening and implode on closing.
    // Also used to change the explode/implode delay.
    inline void MakeItExplode(short ms, short ExpOrImp = expWillExplode)
    {
        if(ExpImpStatus == expNeither)
            ExpImpStatus = ExpOrImp;

        msDelay = ms;
    }

    // Get the current explode/implode delay factor.
    inline short GetDelay(void)
        {   return msDelay;     }

// This stuff will make it streamable.
private:
    virtual const char *streamableName() const
        { return name; }

protected:
    TExplodeDialog(StreamableInit) : TDialog(streamableInit),
        TWindowInit(TExplodeDialog::initFrame) {}
    virtual void write( opstream & );
    virtual void *read( ipstream & );

public:
    static const char * const _NEAR name;
    static TStreamable *build();
};

inline ipstream& operator >> ( ipstream& is, TExplodeDialog& cl )
    { return is >> (TStreamable&)cl; }
inline ipstream& operator >> ( ipstream& is, TExplodeDialog*& cl )
    { return is >> (void *&)cl; }

inline opstream& operator << ( opstream& os, TExplodeDialog& cl )
    { return os << (TStreamable&)cl; }
inline opstream& operator << ( opstream& os, TExplodeDialog* cl )
    { return os << (TStreamable *)cl; }

#endif      // TExplodeDialog
